@@ -1,5 +1,5 @@
 from __future__ import annotations
-from typing import List, Tuple, Optional
+from typing import List, Tuple, Optional, Dict, Set
 from staticsite import Page, Feature
 from staticsite.page import PageNotFoundError
 from staticsite.utils import front_matter
@@ -8,6 +8,7 @@ from staticsite.contents import ContentDir
 from staticsite.utils.typing import Meta
 from urllib.parse import urlparse, urlunparse
 import jinja2
+import markupsafe
 import re
 import os
 import io
@@ -20,14 +21,16 @@ log = logging.getLogger("markdown")
 class LinkResolver(markdown.treeprocessors.Treeprocessor):
     def __init__(self, *args, **kw):
         super().__init__(*args, **kw)
-        self.page = None
-        self.absolute = False
-        self.substituted = {}
+        self.page: Optional[Page] = None
+        self.absolute: bool = False
+        self.substituted: Dict[str, str] = {}
+        self.external_links: Set[str] = set()
 
     def set_page(self, page, absolute=False):
         self.page = page
         self.absolute = absolute
         self.substituted = {}
+        self.external_links = set()
 
     def run(self, root):
         for a in root.iter("a"):
@@ -66,6 +69,9 @@ class LinkResolver(markdown.treeprocessors.Treeprocessor):
 
         # If it's an absolute url, leave it unchanged
         if parsed.scheme or parsed.netloc:
+            self.external_links.add(url)
+            return None, url
+
             return None, parsed
 
         # If it's an anchor inside the page, leave it unchanged
@@ -122,8 +128,8 @@ class StaticSiteExtension(markdown.extensions.Extension):
     def reset(self):
         pass
 
-    def set_page(self, page):
-        self.link_resolver.set_page(page)
+    def set_page(self, page: Page, absolute: bool):
+        self.link_resolver.set_page(page, absolute)
 
 
 class MarkdownPages(Feature):
@@ -148,11 +154,11 @@ class MarkdownPages(Feature):
 
         self.render_cache = self.site.caches.get("markdown")
 
-    @jinja2.contextfilter
+    @jinja2.pass_context
     def jinja2_markdown(self, context, mdtext):
-        return jinja2.Markup(self.render_snippet(context.parent["page"], mdtext))
+        return markupsafe.Markup(self.render_snippet(context.parent["page"], mdtext))
 
-    def render_page(self, page, body, render_type, absolute=False):
+    def render_page(self, page: Page, body: List[str], render_type: str, absolute: bool = False):
         """
         Render markdown in the context of the given page.
         """
@@ -180,6 +186,11 @@ class MarkdownPages(Feature):
         self.markdown.reset()
         rendered = self.markdown.convert("\n".join(body))
 
+        rendered = self.site.metadata.on_contents_rendered(
+                page, rendered,
+                render_type=render_type,
+                external_links=self.link_resolver.external_links)
+
         self.render_cache.put(cache_key, {
             "mtime": page.src.stat.st_mtime,
             "rendered": rendered,
@@ -197,7 +208,7 @@ class MarkdownPages(Feature):
         """
         if content is None:
             content = page.get_content()
-        self.link_resolver.set_page(page)
+        self.link_resolver.set_page(page, absolute=False)
         self.markdown.reset()
         return self.markdown.convert(content)
 
@@ -358,29 +369,32 @@ class MarkdownPage(Page):
     def check(self, checker):
         self.render()
 
-    @jinja2.contextfunction
+    @jinja2.pass_context
     def html_body(self, context, **kw) -> str:
+        absolute = self != context["page"]
         if self.content_has_split:
             body = self.body_start + ["", "<a name='sep'></a>", ""] + self.body_rest
-            return self.mdpages.render_page(self, body, render_type="hb")
+            return self.mdpages.render_page(self, body, render_type="hb", absolute=absolute)
         else:
-            return self.mdpages.render_page(self, self.body_start, render_type="s")
+            return self.mdpages.render_page(self, self.body_start, render_type="s", absolute=absolute)
 
-    @jinja2.contextfunction
+    @jinja2.pass_context
     def html_inline(self, context, **kw) -> str:
+        absolute = self != context["page"]
         if self.content_has_split:
             body = self.body_start + ["", f"[(continue reading)](/{self.src.relpath})"]
-            return self.mdpages.render_page(self, body, render_type="h")
+            return self.mdpages.render_page(self, body, render_type="h", absolute=absolute)
         else:
-            return self.mdpages.render_page(self, self.body_start, render_type="s")
+            return self.mdpages.render_page(self, self.body_start, render_type="s", absolute=absolute)
 
-    @jinja2.contextfunction
+    @jinja2.pass_context
     def html_feed(self, context, **kw) -> str:
+        absolute = self != context["page"]
         if self.content_has_split:
             body = self.body_start + [""] + self.body_rest
-            return self.mdpages.render_page(self, body, render_type="f")
+            return self.mdpages.render_page(self, body, render_type="f", absolute=absolute)
         else:
-            return self.mdpages.render_page(self, self.body_start, render_type="f")
+            return self.mdpages.render_page(self, self.body_start, render_type="f", absolute=absolute)
 
 
 FEATURES = {
