@@ -34,7 +34,7 @@ CONTENT: Optional[str] = None
 
 # Directories where themes are looked for
 THEME_PATHS: Sequence[str] = [
-        os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), "themes")),
+        os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "themes")),
         "/usr/share/staticsite/themes",
 ]
 
